"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cli = void 0;
const chalk_1 = __importDefault(require("chalk"));
const chokidar_1 = __importDefault(require("chokidar"));
const lodash_1 = require("lodash");
const minimist_1 = __importDefault(require("minimist"));
const reselect_1 = require("reselect");
const vorpal_1 = __importDefault(require("vorpal"));
const config_1 = require("./actions/config");
const createStore_1 = require("./createStore");
const getDelimiter_1 = require("./selectors/getDelimiter");
const getHistoryKey_1 = require("./selectors/getHistoryKey");
const createCommonExtension_1 = require("./vorpal/createCommonExtension");
const createRootExtension_1 = require("./vorpal/createRootExtension");
const renderDelimiter_1 = require("./vorpal/renderDelimiter");
const useExtensions_1 = require("./vorpal/useExtensions");
const watchPaths = (paths, handler) => {
    const watcher = chokidar_1.default.watch(paths, {
        ignoreInitial: true,
        awaitWriteFinish: true
    });
    watcher.on('add', handler);
    watcher.on('change', handler);
    watcher.on('unlink', handler);
    return watcher;
};
const watchPathsAndSubscribe = (store, pathsSelector) => {
    const state = store.getState();
    let currentPaths = pathsSelector(state);
    const watcher = watchPaths(currentPaths, () => {
        const state = store.getState();
        store.dispatch(config_1.setConfig(state.config));
    });
    store.subscribe(() => {
        const state = store.getState();
        const previousPaths = currentPaths;
        currentPaths = pathsSelector(state);
        if (previousPaths !== currentPaths) {
            watcher.unwatch(lodash_1.difference(previousPaths, currentPaths));
            watcher.add(lodash_1.difference(currentPaths, previousPaths));
        }
    });
};
const subscribeDelimiter = (store, vorpal) => {
    let currentDelimiter = getDelimiter_1.getDelimiter(store.getState());
    renderDelimiter_1.renderDelimiter(vorpal, currentDelimiter);
    store.subscribe(() => {
        const state = store.getState();
        const previousDelimiter = currentDelimiter;
        currentDelimiter = getDelimiter_1.getDelimiter(state);
        if (previousDelimiter !== currentDelimiter) {
            renderDelimiter_1.renderDelimiter(vorpal, currentDelimiter);
        }
    });
};
const logConfigError = (vorpal, error) => {
    vorpal.log('\n');
    vorpal.log(chalk_1.default.yellow("  Failed to parse Nightwatch configuration and tests. Try fixing the error below and Night Patrol will automatically try again. If that doesn't work, please submit an issue ticket."));
    vorpal.log(chalk_1.default.grey(error));
    vorpal.log('');
};
const subscribeVorpal = (store, vorpal) => {
    let currentConfig = store.getState().config;
    if (!currentConfig.error) {
        vorpal.show();
    }
    else {
        vorpal.hide();
        logConfigError(vorpal, currentConfig.error);
    }
    store.subscribe(() => {
        const state = store.getState();
        const previousConfig = currentConfig;
        currentConfig = state.config;
        if (previousConfig !== currentConfig) {
            if (currentConfig.error && !previousConfig.error) {
                vorpal.hide();
                logConfigError(vorpal, currentConfig.error);
            }
            if (!currentConfig.error && previousConfig.error) {
                vorpal.log('\n  Config parsed successfully. Please continue.\n');
                vorpal.show();
            }
        }
    });
};
const cli = () => {
    const store = createStore_1.createStore();
    const argv = minimist_1.default(process.argv.slice(2));
    store.dispatch(config_1.setConfig({
        configPath: argv.config,
        executablePath: argv.nightwatch,
        currentEnvironment: argv.env
    }));
    const vorpal = new vorpal_1.default();
    vorpal.history(getHistoryKey_1.getHistoryKey(store.getState()));
    useExtensions_1.useExtensions(vorpal, [
        createCommonExtension_1.createCommonExtension(store),
        createRootExtension_1.createRootExtension(store)
    ]);
    watchPathsAndSubscribe(store, reselect_1.createSelector(state => state.config.configPath, configPath => [configPath]));
    watchPathsAndSubscribe(store, state => state.config.suiteDirectories);
    subscribeDelimiter(store, vorpal);
    subscribeVorpal(store, vorpal);
};
exports.cli = cli;
//# sourceMappingURL=cli.js.map